@@ -15,9 +15,9 @@ Readonly my $FULLPAGE_OCR_SCALE => 0.8;
 my $SPACE = q{ };
 my $EMPTY = q{};
 my $device;
-my @old_idles;
+my %old_idles;
 
-our $VERSION = '2.6.4';
+our $VERSION = '2.6.5';
 
 use Glib::Object::Subclass GooCanvas2::Canvas::, signals => {
     'zoom-changed' => {
@@ -118,16 +118,14 @@ sub set_text {
         $idle = TRUE;
     }
     my $root;
-    if (@old_idles) {
-        while (@old_idles) {
-            Glib::Source->remove( pop @old_idles );
+    if (%old_idles) {
+        while ( my ( $box, $source ) = each %old_idles ) {
+            Glib::Source->remove($source);
+            delete $old_idles{$box};
         }
-        $root = GooCanvas2::CanvasGroup->new;
-        $self->set_root_item($root);
-    }
-    else {
-        $root = $self->get_root_item;
     }
+    $root = GooCanvas2::CanvasGroup->new;
+    $self->set_root_item($root);
     if ( not defined $page->{w} ) {
 
         # quotes required to prevent File::Temp object being clobbered
@@ -147,16 +145,25 @@ sub set_text {
 
     # Attach the text to the canvas
     for my $box ( @{ $page->boxes } ) {
+        my %options = (
+            root           => $root,
+            box            => $box,
+            transformation => [ 0, 0, 0 ],
+            edit_callback  => $edit_callback,
+            text_color     => $color_hex,
+            idle           => $idle,
+        );
         if ($idle) {
-            push @old_idles, Glib::Idle->add(
+            $old_idles{$box} = Glib::Idle->add(
                 sub {
-                    _boxed_text( $root, $box, [ 0, 0, 0 ],
-                        $edit_callback, $color_hex );
+                    _boxed_text(%options);
+                    delete $old_idles{$box};
+                    return Glib::SOURCE_REMOVE;
                 }
             );
         }
         else {
-            _boxed_text( $root, $box, [ 0, 0, 0 ], $edit_callback, $color_hex );
+            _boxed_text(%options);
         }
     }
     return;
@@ -215,7 +222,13 @@ sub get_offset {
 # Draw text on the canvas with a box around it
 
 sub _boxed_text {
-    my ( $root, $box, $transformation, $edit_callback, $text_color ) = @_;
+    my (%options)      = @_;
+    my $root           = $options{root};
+    my $box            = $options{box};
+    my $transformation = $options{transformation};
+    my $edit_callback  = $options{edit_callback};
+    my $text_color     = $options{text_color};
+    my $idle           = $options{idle};
     my ( $rotation, $x0, $y0 ) = @{$transformation};
     my ( $x1, $y1, $x2, $y2 ) = @{ $box->{bbox} };
     my $x_size = abs $x2 - $x1;
@@ -311,8 +324,26 @@ sub _boxed_text {
     }
     if ( $box->{contents} ) {
         for my $box ( @{ $box->{contents} } ) {
-            _boxed_text( $g, $box, [ $textangle + $rotation, $x1, $y1 ],
-                $edit_callback, $text_color );
+            my %noptions = (
+                root           => $g,
+                box            => $box,
+                transformation => [ $textangle + $rotation, $x1, $y1 ],
+                edit_callback  => $edit_callback,
+                text_color     => $text_color,
+                idle           => $idle,
+            );
+            if ($idle) {
+                $old_idles{$box} = Glib::Idle->add(
+                    sub {
+                        _boxed_text(%noptions);
+                        delete $old_idles{$box};
+                        return Glib::SOURCE_REMOVE;
+                    }
+                );
+            }
+            else {
+                _boxed_text(%noptions);
+            }
         }
     }
 
