@@ -296,7 +296,7 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::, signals => {
     ),
   ];
 
-our $VERSION = '2.6.4';
+our $VERSION = '2.6.5';
 
 my ( $d_sane, $logger );
 my $SANE_NAME_SCAN_TL_X   = SANE_NAME_SCAN_TL_X;
@@ -377,6 +377,10 @@ sub INIT_INSTANCE {
                 $bscanall->set_active(TRUE);
             }
             else {
+                # if spin button is already $value, but pages = all is selected,
+                # then the callback will not fire to activate # pages, so doing
+                # it here
+                $bscannum->set_active(TRUE);
                 $spin_buttonn->set_value($value);
             }
 
@@ -537,6 +541,9 @@ sub INIT_INSTANCE {
             my ( $widget, $value ) = @_;
             $self->set( 'page-number-increment',
                 $value eq 'facing' ? $DOUBLE_INCREMENT : -$DOUBLE_INCREMENT );
+            if ( $value eq 'facing' ) {
+                $self->set( 'num-pages', 0 );
+            }
         }
     );
     $self->{combobs}->set_tooltip_text(
@@ -2397,6 +2404,7 @@ sub add_current_scan_options {
 
 sub _set_option_profile {
     my ( $self, $profile, $next, $step ) = @_;
+    $self->set( 'cursor', 'wait' );
 
     if ( my $i = $next->($step) ) {
         my ( $name, $val ) = $profile->get_backend_option_by_index($i);
@@ -2489,6 +2497,7 @@ sub _set_option_profile {
             $self->get('current-scan-options'),
             $profile->{uuid}
         );
+        $self->set( 'cursor', 'default' );
     }
     return;
 }
